// C Source File
// Created 6/25/00; 4:45:57 PM

/* 

Disclaimer:

This is my first large C program, so I'm still getting used to the language.  This program doesn't 
pretend to have great coding style.  However, it works! :-)


*/

#define LINK_SEND_PIECE 0x44
#define LINK_END_GAME 0x33
#define WIN  5
#define LOSE 6
#define TEACHER 7
#define PAUSE 8 // these are various constants that I use throughout the program, it's easier to remember
                // what they are if I give them names

#define SAVE_SCREEN
#define OPTIMIZE_ROM_CALLS // I like this directive a lot!  Move the cursor onto it and press F1 to get an
                           // explanation of what it does.  Note that you can do this for _ANY_ TI-GCC 
                           // library call I use in this program.  I suggest you do this a lot if you 
                           // are just learning TI-GCC.

#include <nostub.h>  // Make NoStub Program
#include <graph.h>   // Support Graphics
#include <kbd.h>     // The keyboard routines
#include <asmtypes.h>// defines some special types
#include <sprites.h> // for sprite8 sprite writing routine
#include <mem.h>     // for memcpy, memset, memcmp (remember that you can press F1 on these to see what
                     // they are if you don't know)
#include <system.h>  // For OSSetSR (disabling and enabling of interrupts), OSCheckBreak (checks the ON key)
#include <stdlib.h>  // random number routines
#include <textedit.h>// I use the text editor to allow people to enter their names for the high score screen
#include <events.h>  // for EV_getc function to work with text editor
#include <string.h>  // for strcat and strcpy
#include <Wingraph.h>// for DeskTop window
#include <link.h>    // for link routines
#include <intr.h>    // for interrupt handling routines
#include <gray.h>    // for grayscale routines
#include "titlescreen.h" // I put the title screen picture in a different file because it's a giant array.
                         // if you want to know how to create and display title screen pictures of your 
                         // own, or you want to convert some images you have to sprites or bitmaps so you
                         // can display them on your calculator, go to tict.ticalc.org (no www in front)
                         // and look at their tutorial on handling images in C.
#include <alloc.h>   // for HeapAllocPtr


int _ti89;           // Produce .89Z File

typedef volatile unsigned char PIECE[3][3]; // this defines a PIECE to be a 3x3 array of characters (bytes).
                                            // Volatile means that a PIECE can be changed through the
                                            // action of an interrupt or something that can't be predicted
                                            // just from reading the source code
                                            
                                            
// I have _way_ too many global variables.  Many probably don't need to be global, but I'm too lazy to 
// change them now.

PIECE currpiece;  // current piece on the board

unsigned char sqsp[6] = {0xFC,0xFC, 0xFC, 0xFC, 0xFC, 0xFC};  // sprite of a 6*6 pixel square

unsigned char squarez[3][3][5] = {{{0xff,0xd6,0xe6,0xf0,0xa0},{0xff,0xDE,0x7f,0xfc,0xe4},{0xdf,0x8f,0x73,0x68,0x40}},{{0xf6,0xBB,0xfa,0x2f,0x78},{0xef,0xff,0x9f,0xdf,0xfe},{0xf5,0x7d,0xfc,0x17,0x9a}},{{0xda,0xe9,0xc3,0x22,0x08},{0xaf,0x66,0x19,0xdd,0x06},{0x79,0x31,0xa4,0x82,0x10}}};
// this is the binary data for each squarez piece.  See the getpiece() function.  Thanks to Jimmy Mardell for letting me use this data from his program Squarez for the 86. (I have no idea where he got it.  It sure was a pain to translate this into hex from binary by hand!!!)

unsigned long score; // the score.  duh! :-)

unsigned char board[16][16]; // the entire SquareZ board.

unsigned long highscores[5] = {5000,4000,3000,2000,1000}; // the highscores.

char highnames[5][4] = {"---","---","---","---","---"}; // the names of the people with the high scores

PIECE badpiece = {{1,0,1},{0,1,0},{1,0,1}};      // this is the "bad piece" you get sent in 
                                                 // two player games - the piece format
                                                 // is a 3*3 array of ones and zeros

volatile int timer = 10, counter = 0; // these are used to run the game timer
volatile int x = 0, y = 0;            // x and y coordinates of the current moving piece
volatile char twoplay = 0;            // 0 if one player, 1 if two player
PIECE nextpiece;                      // the next piece (displayed in the "next" box)
volatile unsigned int movedelay = 0,rotatedelay = 0;  // these are used to keep the pieces from rotating and moving too fast
volatile char gameover = 0;           // this notifies the _main routine when the interrupt routine discovers that 
                                      // the game has ended for whatever reason.

volatile int intcounter = 0;          // is incremented every auto-int-1, when it reaches 18, gameloop() 
                                      // is called
int gameloopDelay = 18;               // this is used to compensate for the slower interrupts on HW2 calcs.

// End of the global variables.  Now we start defining functions

unsigned long GetHardwareVersion ()
{
  unsigned long hwpb, *rombase;
  rombase = (unsigned long *)((*(unsigned long *)0xC8) & 0x600000);
  hwpb = rombase[65];
  return (hwpb - (unsigned long)rombase  < 0x10000 &&
    *(unsigned short *)hwpb > 22 ? *(unsigned long *)(hwpb + 22) : 1);
}


void square(int y,int x)              // draws a single square at board coordinates x,y
{
	Sprite8(6*x+2,6*y+2,6,sqsp,LCD_MEM,SPRT_XOR);
};

void getpiece (int num, PIECE piece)   // gets a piece from the array "squarez" and puts it into the
{                                      // 3*3 PIECE array piece
	int x=0,y;
	for (; x < 3;x++) // these two nested for loops step through the 3x3 array that defines a piece.
                      // You'll see a lot of nested for loops like these in this program.
	{
		for (y = 0; y < 3; y++)
		{
 			piece[x][y] = ((0x80 >> (num % 8)) & (squarez[x][y][num / 8])) ? 1 : 0; // This is the heart of the routine, but it's rather cryptic and hard to explain.  If you can't figure it out, you don't really need to know.
		};
	};
};

void drawpiece (int x, int y, PIECE piece) // draws the piece "piece" at coordinates x and y on the
{                                          // board - it can erase pieces too, simply by drawing the
	int x1=0,y1;
	for (; x1 < 3;x1++)
	{
		for (y1 = 0; y1 < 3; y1++) // step through the piece array
		{
			if (piece[x1][y1] == 1 )  // if there's a square at x1,y1
				square(x+x1,y+y1); // draw a square there
		};
	};
};

int checkpiece (int x, int y, PIECE piece) // checks to see if the given piece at x and y overlaps any
{                                          // squares already on the board, or goes over the edge
	int x1=0,y1;                           // returns 0 if the piece can't fit
	for (; x1 < 3;x1++)
	{
		for (y1 = 0; y1 < 3; y1++) // step through the piece
		{
			if (piece[x1][y1] == 1 ) // If there's a square at x1,y1 in the piece
			{
				if (x1 + x > 15 || y1 + y > 15 || x1 + x < 0 || y1 + y < 0) // if we're off the edge of the board
					return 0; //
				if (board[x1+x][y1+y] == 1) // if there's already a square on the board here
					return 0;
			};
		};
	};
	return 1; // we're through checking, the piece will fit
};

int droppiece (int x, int y, PIECE piece)  // checks to see if the piece can be dropped - if not, 
{                                          // returns 0.  Otherwise, it sets the board array to indicate
	int x1=0,y1;                           // that the piece has been dropped and returns 1
	for (; x1 < 3;x1++)
	{
		for (y1 = 0; y1 < 3; y1++) // these for loops step through the 3x3 array of the piece
		{
			if (piece[x1][y1] == 1 ) // if the piece has a square at position x1, y1
			{			
				if (board[x1+x][y1+y] == 2) // if that square on the board happens to be a forbidden square (at the top left)
					return 0;               // we can't drop the piece here
			};
		};
	}; // if we've gotten here, the piece can be dropped
	for (x1 = 0; x1 < 3;x1++)
	{
		for (y1 = 0; y1 < 3; y1++) // stepping through the piece's array again
		{
			if (piece[x1][y1] == 1 ) // if the piece has a square here
			{
				board[x1+x][y1+y] = 1; // set the board to indicate that there's a square here now
			};
		};
	};
	return 1;
};

void clearboard()  // clears the board array but not the screen
{
	int x1;
	memset(board,0,256); // sets the entire array to 0 all at once!
	for (x1 = 0; x1 < 3;x1++) // this for loop marks the top left with 2s to indicate that you can't drop a piece there
	{
		memset(board[x1],2,3);
	};
};

void rotateright (PIECE piece)  // rotates the given piece to the right
{
	PIECE temppiece = {}; // temporary piece that will be copied back to piece in the end
	int x1 = 0, y1;
	for (; x1 < 3;x1++) 
	{
		for (y1 = 0; y1 < 3; y1++) // step through the piece array
		{
			temppiece[y1][2 - x1] = piece[x1][y1]; // this is hard to explain - if you run through these loops by hand you'll understand what's happening
		};
	};
	memcpy(piece,temppiece,9); // the temporary piece has been rotated now, so we'll copy it over to the original piece
};

void rotateleft (PIECE piece)  // rotates the given piece to the left - see rotateright
{
	PIECE temppiece = {};
	int x1 = 0, y1;
	for (; x1 < 3;x1++)
	{
		for (y1 = 0; y1 < 3; y1++)
		{
			temppiece[2 - y1][x1] = piece[x1][y1];
		};
	};
	memcpy(piece,temppiece,9);
};

void erasesquarez()  // this large routine checks the board for 3*3 squares and marks them, then later goes
{					 // back and erases them from the board and the screen.  If two player mode is active, 
					 // it sends a piece to the other player if needed.  It also updates the score.
	int x1=0,y1,x2,y2,somesquarez = 0, count = 0;
	char buffer[10];
	buffer[0] = LINK_SEND_PIECE; 
	for (; x1 < 14;x1++)
	{
		for (y1 = 0; y1 < 14; y1++)
		{
			for (x2 = 0; x2 < 3; x2++)
			{
				for (y2 = 0; y2 < 3; y2++)
				{
					if (board[x1+x2][y1+y2] == 0 || board[x1+x2][y1+y2] == 2)
						goto nosquare;
				};
			};
			for (x2 = 0; x2 < 3; x2++)
			{
				for (y2 = 0; y2 < 3; y2++)
				{
					if (board[x1+x2][y1+y2] != 3)
					{
						square(x1+x2,y1+y2);
						board[x1+x2][y1+y2] = 3;
					}
					else
					{
						score = score + 50;
					};
				};
			};
			count++;
			if(twoplay && count == 2)
			{
				OSWriteLinkBlock(buffer,1);
			}
			somesquarez = 1;
			score = score + 100;
			nosquare:
		};
	};
	if (somesquarez)
	{
		buffer[0] = 1;
		for (x1 = 0; x1 < 16;x1++)
		{
			for (y1 = 0; y1 < 16; y1++)
			{
				if (board[x1][y1] == 3)
				{
					board[x1][y1] = 0;
					count++;
				}
				if (board[x1][y1] != 0 && board[x1][y1] != 2)
					buffer[0] = 0;
			};
		};
//		if (1)//(count > 1) && twoplay)
//		{
//			OSWriteLinkBlock(buffer,1); // send the contents of buffer (LINK_SEND_PIECE)
//		}
		if (buffer[0] == 1)
		{
			score += 3000;
			DrawStr(103,60,"3000 pt. Bonus!",A_NORMAL);  // a little notification
		};
	};
};

void TestHighScore()  // this routine checks to see if you got a high score.  If you did, it starts the
{					  // text editor so you can type your name and then updates the high score table.
	int x = 0;
	char buffer[4] = "\0\0\0";
	for(;x < 5; x++)
	{
		if (highscores[x] < score)
		{
			int y = 5;
			EVENT ev;
			TEXT_EDIT te;
			ClrScr();
			FontSetSys(F_6x8);
			DrawStr(1,1,"Congratulations!",A_NORMAL);
			DrawStr(1,12,"You got a high score!",A_NORMAL);
			DrawStr(1,23,"Enter your initials:",A_NORMAL);
			TE_openFixed (&te, DeskTop, MakeWinRect (1, 34, 130, 70), buffer, 3, 0);
			TE_select(&te,0,0);
			TE_focus(&te);
			while (EV_getc (ACTIVITY_BUSY, &ev) != 13)
			{
				if (ev.extra.Key.Code == 264)
					break;
				TE_handleEvent (&te, &ev);
			};
			while (--y > x)
			{
				highscores[y] = highscores[y - 1];
				strcpy(highnames[y],highnames[y-1]);
			};
			highscores[x] = score;
			strcpy(highnames[x],buffer);
			TE_close(&te);
			break;
		};
	};
};



//       |*|
//       |*|   LINK CODE HERE
//      \|*|/  
//       \*/

int connect()  // this function attempts to establish link communication with another calculator.
{
	int y;
	char buf2[2];
	char buffer[10];
	void connected()  // called when connection is established
	{
		DrawStr(1,24,"Connection Established!",A_NORMAL);
		DrawStr(1,36,"Press any key to start.",A_NORMAL);
		OSReadLinkBlock(buffer,5);  // Make sure the receive buffer is clear
		buffer[0] = 0;  // clear buffer
		while (!( _rowread(0x80) & 0x3F) && !buffer[0])
			OSReadLinkBlock(buffer,1);  // wait for the other calculator to start or until the user presses a button (if I didn't test the buttons, this might go into an endless loop!  The _rowread() statement allows a way to get out if something goes wrong)
		OSWriteLinkBlock(buf2, 1);  // inform the other calculator we're starting
	};
	
	ClrScr();
	OSClearBreak();
	DrawStr(1,1,"Connecting...",A_NORMAL);
	OSLinkOpen();	  // Open link communication
	buf2[0] = 0x22;   // 0x22 is the message I will send out over the link - one byte
	OSWriteLinkBlock( buf2,1);  // send 0x22 over the link - if this is the first calculator to send, this byte will be lost.  It doesn't matter though, because the second calculator's byte will be received by the first calculator later on.
	for (y = 0; y < 30000;y++);  // wait a bit
	OSReadLinkBlock(buffer,1);  // check for a response
	if (buffer[0] == 0x22) //  If we've been sent the 0x22 byte
	{
		connected(); // yay!
		return 1;
	}
	DrawStr(1,12,"Press ON to stop.",A_NORMAL);  // we didn't get a response, so now we'll wait for someone to contact us
	while (!OSCheckBreak())  // Until ON is pressed
	{
		OSReadLinkBlock(buffer,1); // see what has been sent to us - buffer will be zero if nothing has been sent
		if (buffer[0] == 0x22)     // if we've been sent the 0x22 byte
		{	
			OSWriteLinkBlock(buf2,1); // reply
			connected();
			return 1; // yay!  we connected so we can return 1
		};
	};
	ClrScr();
	OSClearBreak();
	DrawStr(1,1,"No Connection.",A_REPLACE);  // Darn!  We didn't connect in the while loop
	while (!( _rowread(0x80) & 0x3F) && !OSCheckBreak());  // wait for the user to see the message
	while (( _rowread(0x80) & 0x3F)); // wait for the user to release the key he/she just pressed
	GKeyFlush();
	return 0;  // return 0 because we didn't connect
};

volatile void gameloop()  // the actual game loop! (well, it's sort of like a loop...)  This is called from auto-int 1.
{
	volatile unsigned char key = 0;
	char buffer[10];
//	OSSetSR(0x0300);
	if (gameover == PAUSE)
	{
		if (twoplay)
		{
			buffer[0] = 0;
			OSReadLinkBlock(buffer,1);
			if (buffer[0] == PAUSE)
				goto unpause;
		};
		if (_rowread(0xFE) & 0x80)
		{
			if (twoplay)
			{
				buffer[0] = PAUSE;
				OSWriteLinkBlock(buffer,1);
			}
			OSSetSR(0x0700);
			unpause:
			gameover = 0;
			while (_rowread(0xFE) & 0x80);
			FontSetSys(F_4x6);
			DrawStr(112, 3,"                          ",A_REPLACE);
			FontSetSys(F_8x10);
			OSSetSR(0x0000);
		};
	}
	if (gameover != 0)
		return;
	counter++;
//       |*|
//       |*|   LINK CODE HERE
//      \|*|/  
//       \*/
	if (twoplay)  // if in 2-player mode
	{
		buffer[0] = 0;
		OSReadLinkBlock(buffer,1);  // see if anything has been sent to us
		if (buffer[0] == LINK_SEND_PIECE)  // if the other calculator has sent us a piece
		{
			if (checkpiece(x,y,badpiece) && memcmp(currpiece,badpiece,6)) // if the current piece can safely be replaced by the bad piece and it is not already the bad piece
			{
				drawpiece (x,y,currpiece); // replace the current piece with the bad piece
				memcpy(currpiece, badpiece, 9);
				drawpiece (x,y,currpiece);
			}
			else  // otherwise, put the bad piece into the next piece slot
			{
				drawpiece (6,19,nextpiece);     // erase the "next" piece display
				memcpy(nextpiece,badpiece,9);   // make the next piece the "bad" piece
				drawpiece (6,19,nextpiece);     // draw the "bad" piece in the next piece display
			};
			FontSetSys(F_4x6);
			DrawStr(103,60,"The Dreaded X!",A_NORMAL);  // a little notification
			FontSetSys(F_8x10);
//			OSSetSR (0x0300);
		};
		if (buffer[0] == LINK_END_GAME)  // if the other calculator's game has finished
		{
			gameover = WIN;  // we've won!
			return;
		};
		if (buffer[0] == TEACHER)
		{
			gameover = TEACHER;
			return;
		};
		if (buffer[0] == PAUSE)
		{
			gameover = PAUSE;
			FontSetSys(F_4x6);
			DrawStr(112,3,"Paused", A_NORMAL);
			return;
		}
	};
	if (counter == 15)  // every 15th time gameloop is called
	{
		counter = 0;  // reset the counter
		timer--;      // decrement the timer
		if (timer < 0)// if time is up
		{
			if (droppiece(x,y,currpiece))  // if the piece can be dropped
			{
				memcpy(currpiece,nextpiece,9);  // do a bunch of stuff to get ready for the next piece
				drawpiece(6,19,nextpiece);      //
				getpiece (random(39),nextpiece);
				drawpiece(6,19,nextpiece);
				x = 0; y = 0;
				FontSetSys(F_4x6);
				DrawStr(103,60,"                          ",A_REPLACE);
				erasesquarez();
				drawpiece(x,y,currpiece);
				counter = 0;
				timer = 9;
				sprintf(buffer,"%lu",score);
				DrawStr(126,82,buffer, A_REPLACE);
				if (score > highscores[0])
					DrawStr(101,91,"NEW HIGH SCORE!",A_REPLACE);
				FontSetSys(F_8x10);
				sprintf(buffer,"%d",timer );
				DrawStr(120,10,buffer, A_REPLACE);
				//OSSetSR (0x0300);
			}
			else  // the piece can't be dropped!  Game over!
			{
				gameover = 1;
//               |*|
//               |*|   LINK CODE HERE
//              \|*|/  
//               \*/

				if (twoplay)
				{
					buffer[0] = LINK_END_GAME;
					OSWriteLinkBlock(buffer,1);
				};
				return;
			};
		};
		sprintf(buffer,"%d",timer);  // update the timer on the screen
		DrawStr(120,10,buffer, A_REPLACE);
		///OSSetSR (0x0300);
	};
	if (_rowread(0xBF) & 0x01)  // if ESC has been pressed
	{
		gameover = LOSE;  // game over
		if (twoplay)      // if in two player mode
		{
			buffer[0] = LINK_END_GAME; // notify the other calc
			OSWriteLinkBlock(buffer,1); //
		};
		return;
	};
	if (_rowread(0x77) & 1)
	{
		gameover = TEACHER;
		if (twoplay)
		{
			buffer[0] = TEACHER;
			OSWriteLinkBlock(buffer,1);
		}
		return;
	}
	key = (int) _rowread(0xFE);  // check the state of the keys
	if (movedelay == 0)  // I may comment the rest of this function someday...
	{
		if (key & 0x02)
		{
			movedelay = 1;
			if (checkpiece (x,y-1,currpiece) == 1)
			{
				drawpiece(x,y,currpiece);
				drawpiece(x,--y,currpiece);
			};
		};
		if (key & 0x04)
		{
			movedelay = 1;
			if (checkpiece (x + 1,y,currpiece) == 1)
			{
				drawpiece(x,y,currpiece);
				drawpiece(++x,y,currpiece);
			};
		};
		if (key & 0x08)
		{
			movedelay = 1;
			if (checkpiece (x,y+1,currpiece) == 1)
			{
				drawpiece(x,y,currpiece);
				drawpiece(x,++y,currpiece);
			};
		};
		if (key & 0x01)
		{
			movedelay = 1;
			if (checkpiece (x-1,y,currpiece) == 1)
			{
				drawpiece(x,y,currpiece);
				drawpiece(--x,y,currpiece);
			};
		};
	}
	else
	{
		movedelay--;
	}
	if (rotatedelay == 0)
	{
		if (key & 0x20)
		{
			PIECE temppiece;
			rotatedelay = 4;
			memcpy(temppiece, currpiece, 9);
			rotateright(currpiece);
			if (checkpiece (x,y,currpiece) == 1)
			{
				drawpiece(x,y,temppiece);
				drawpiece(x,y,currpiece);
			}
			else
			{
			memcpy(currpiece, temppiece, 9);
			};
		};
		if (key & 0x10)
		{
			PIECE temppiece;
			rotatedelay = 4;
			memcpy(temppiece, currpiece, 9);
			rotateleft(currpiece);
			if (checkpiece (x,y,currpiece) == 1)
			{
				drawpiece(x,y,temppiece);
				drawpiece(x,y,currpiece);
			}
			else
			{
				memcpy(currpiece, temppiece, 9);
			};
		};
	}
	else
	{
		if (key & 0x30)
		{rotatedelay--;}
		else
		{rotatedelay = 0;};
	};
	if (key & 0x40)
	{
		if (droppiece(x,y,currpiece))
		{
			score = score + 11;
			memcpy(currpiece,nextpiece,9);
			drawpiece(6,19,nextpiece);
			getpiece (random(39),nextpiece);
			drawpiece(6,19,nextpiece);
			x = 0; y = 0;
			FontSetSys(F_4x6);
			DrawStr(103,60,"                          ",A_REPLACE);
			erasesquarez();
			drawpiece(x,y,currpiece);
			counter = 0;
			timer = 9;
			if (score > highscores[0])
				DrawStr(101,91,"NEW HIGH SCORE!",A_REPLACE);
			sprintf(buffer,"%lu",score);
			DrawStr(126,82,buffer, A_REPLACE);
			FontSetSys(F_8x10);
			sprintf(buffer,"%d",timer);
			DrawStr(120,10,buffer, A_REPLACE);
			//OSSetSR (0x0300);
		};
	};
	if (key & 0x80)
	{
		gameover = PAUSE;
		FontSetSys(F_4x6);
		DrawStr(112,3,"Paused", A_NORMAL);
		OSSetSR(0x0700);
		while (_rowread(0xFE) & 0x80)
			continue;
		OSSetSR(0x0000);
		if (twoplay)
		{
			buffer[0] = PAUSE;
			OSWriteLinkBlock(buffer,1);
		}
		return;
	}
};


DEFINE_INT_HANDLER (int1)  // interrupt handler
{
	intcounter++;
	if (intcounter == gameloopDelay)
	{
		intcounter = 0;
		gameloop();
	};
};
// display the light gray backround of squarez

void background(void * lightplane)
{
	PIECE temp;
	int x,y;
	void drawlightpiece (int x3, int y3, PIECE piece) // draws the piece "piece" at coordinates x and y on the
	{												// board - uses xor so it can erase pieces too
		int x1=0,y1=0;
		for (; x1 < 3;x1++)
		{
			for (y1 = 0; y1 < 3; y1++)
			{
				if (piece[x1][y1] == 1 )
					Sprite8(6*(x3 + x1) +2 , 6*(y3 + y1)+2 , 6 , sqsp , lightplane , SPRT_XOR );
			};
		};
	};
	for (x = 0; x <= 24; x += 3)
	{
		for (y = 0; y <= 15; y += 3)
		{
			getpiece(random(39), temp);
			drawlightpiece(x,y, temp);
		};
	};
};

void DispHighScores()  // displayes the high scores screen
{
	int x = 0;
	void * lightplane;
	void * darkplane;
	char buffer[10];
	void drawscores()
	{
		DrawStr(46,4,"High ScoreZ",A_NORMAL);
		for (x = 0;x < 5; x++)
		{
			sprintf(buffer,"%d",x+1);
			DrawStr(40,11*x+27,buffer, A_NORMAL);
			sprintf(buffer,"%ld",highscores[x]);
			DrawStr(54,11*x + 27,buffer, A_NORMAL);
			DrawStr(100,11*x+27,highnames[x], A_NORMAL);
		};
	};	
	FontSetSys(F_6x8);
	GrayMode(GRAY_ON);
	lightplane = GetPlane(LIGHT_PLANE);
	darkplane = GetPlane(DARK_PLANE);
	memset(darkplane,0,3000);
	memset(lightplane,0,3000);
	background(lightplane);
	PortSet (lightplane, 239, 127);
	drawscores();
	PortSet (darkplane, 239, 127);
	drawscores();
	PortRestore();
	GKeyFlush();	
	ngetchx();
	GrayMode(GRAY_OFF);
};

// display title screen

int titlemenu()
{
	char * darkplane;
	char * lightplane;
	int x = 0;
	
/*	void drawlightpiece (int x3, int y3, PIECE piece) // draws the piece "piece" at coordinates x and y on the
	{												// board - uses xor so it can erase pieces too
		int x1=0,y1=0;
		for (; x1 < 3;x1++)
		{
			for (y1 = 0; y1 < 3; y1++)
			{
				if (piece[x1][y1] == 1 )
					Sprite8(6*(x3 + x1) +2 , 6*(y3 + y1)+2 , 6 , sqsp , lightplane , SPRT_XOR );
			};
		};
	};
*/	GrayMode(GRAY_ON);
	darkplane = (char *) GetPlane(DARK_PLANE);
	lightplane = (char *) GetPlane(LIGHT_PLANE);
	memset(darkplane,0,3000);
	memset(lightplane,0,3000);
/*	for (; x < 100; x++)
	{
		memcpy(darkplane + 30*x, &titlescreen[x*20], 20);
		memcpy(lightplane + 30*x, &titlescreen[2000 + x*20], 20); 
	};*/
/*	for (x = 0; x <= 24; x += 3)
	{
		for (y = 0; y <= 15; y += 3)
		{
			getpiece(random(39), temp);
			drawlightpiece(x,y, temp);
		}
	}
*/	background(lightplane);
	FontSetSys(F_4x6);
	PortSet (darkplane, 239, 127); 
	BitmapPut (18, 20, &titlelayer2, ScrRect, A_NORMAL); 
	DrawStr(18,47,"Version 2.0", A_NORMAL);
	DrawStr(25, 69, "Press T for two-player link mode", A_NORMAL);
	DrawStr(43, 81, "Any other key to start", A_NORMAL);
	PortSet (lightplane, 239, 127); 
	BitmapPut (18, 20, &titlelayer2, ScrRect, A_NORMAL); 
	DrawStr(18,47,"Version 2.0", A_NORMAL);
	DrawStr(25, 69, "Press T for two-player link mode", A_NORMAL);
	DrawStr(43, 81, "Any other key to start", A_NORMAL);
	PortRestore();
	FontSetSys(F_6x8);
	x = ngetchx();
	while (( _rowread(0x80) & 0x3F));
	GrayMode(GRAY_OFF);
	return x;
	//////////////////////////////////////////////////////////////////////////////////////////////////////
}

// Main Function
void _main()
{
	char buffer[10];
	INT_HANDLER save_int_1;
	save_int_1 = GetIntVec(AUTO_INT_1);
	randomize();
	if (GetHardwareVersion() == 1)
	{
		gameloopDelay = 18;
	}
	else
	{
		gameloopDelay = 12;
	};
	//OSSetSR(0x0700);
	GKeyFlush();  // remove all previous keystrokes from the keyboard buffer
	if (titlemenu() == 116)  //  if the user pressed "T" at the title screen
	{
		if (connect() == 1)  // try to connect - if successful, then...
		{
			twoplay = 1;     // set a flag variable saying that we're in two player mode
		}
		else         // if we didn't connect (connect == 0)
		{
			exit(0);
		};
	}; // now twoplay is set to the correct value and the link is ready if in two player mode	

// Drawing the board...
	ClrScr();
	DrawLine(1,1,1,98,A_NORMAL);
	DrawLine(1,98,98,98,A_NORMAL);
	DrawLine(98,98,98,1,A_NORMAL);
	DrawLine(98,1,1,1,A_NORMAL);
	DrawLine(2,2,19,19,A_NORMAL);
	DrawLine(2,19,19,2,A_NORMAL);
	DrawLine(114,36,135,36,A_NORMAL);
	DrawLine(114,36,114,57,A_NORMAL);
	DrawLine(114,57,135,57,A_NORMAL);
	DrawLine(135,57,135,37,A_NORMAL);
	DrawStr(113,27,"Next",A_NORMAL);
	FontSetSys(F_4x6);
	DrawStr(102,82,"Score: 0",A_NORMAL);
	DrawStr(102,91,"High:",A_NORMAL);
	sprintf(buffer,"%lu",highscores[0]);
	DrawStr(126,91,buffer,A_NORMAL);
	DrawClipRect(MakeWinRect(101,66, 157, 77), ScrRect, A_NORMAL);
	FontSetSys(F_8x10);
	DrawStr(101,67,"SquareZ",A_REVERSE);
	clearboard();
	score = 0;
	getpiece (random(39),currpiece);
	getpiece (random(39),nextpiece);
	drawpiece(6,19,nextpiece);
	drawpiece(x,y,currpiece);
	DrawStr(120,10,"9", A_REPLACE);
	SetIntVec(AUTO_INT_1, int1); // install the interrupt
	//OSSetSR (0x0300);
	for (;;) // (nearly) endless loop!
	{
		if (gameover != PAUSE && gameover != 0)
		{
			break;
		};
		idle(); // Use less batteries!
//		OSSetSR (0x0300);
	};
	SetIntVec(AUTO_INT_1, save_int_1); // remove the interrupt
	OSSetSR (0x0000); 
	
	if (gameover == TEACHER)
		goto teacher;
	
	drawpiece(0,0,nextpiece);
	FontSetSys(F_8x10);
	if (twoplay)
	{
		unsigned long * otherscore = HeapAllocPtr(4);
		char * back;
		char buf2[40];
//		OSReadLinkBlock(( char *) otherscore,4);
//		OSReadLinkBlock(( char *) otherscore,4);
//		OSReadLinkBlock(( char *) otherscore,4);
//		for (x = 0; x < 20000; x++);
		OSWriteLinkBlock((char *) &score,4);
		for (x = 0; x < 20030; x++);
		OSReadLinkBlock(( char *) otherscore,4);
		if (gameover == WIN)
		{
			DrawStr(17,42,"You won!",A_REPLACE);
			if (score < *otherscore)
			{
				strcpy(buf2,"but you got ");
				back = " less points.";
				*otherscore = *otherscore - score;
			}
			else
			{
				strcpy(buf2,"and you got ");
				back = " more points!";
				*otherscore = score - *otherscore;
			}
		}
		else
		{
			DrawStr(16,42,"You lost.",A_REPLACE);
			if (score <= *otherscore)
			{
				strcpy(buf2,"and you got ");
				back = " less points.";
				*otherscore = *otherscore - score;
			}
			else
			{
				strcpy(buf2,"but you got ");
				back = " more points!";
				*otherscore = score - *otherscore;
			}
		}
		sprintf(buffer, "%ld", *otherscore);
		FontSetSys(F_4x6);
		DrawStr(2,53,strcat(strcat(buf2,buffer),back),A_REPLACE);
		DrawLine(2,52,97,52,A_REVERSE);
		DrawLine(2,58,97,58,A_REVERSE);
		HeapFreePtr(otherscore);
		for (x = 0; x < 25000;x++)
			for (y = 0; y < 5; y++); // wait for a while - we don't want the user skipping the end screen by accident
	}
	else
	{
		DrawStr(14,45,"Game Over", A_REPLACE);
	}
	while (( _rowread(0x80) & 0x3F)); // wait for the user to release any keys he/she is pressing
	GKeyFlush();
	ngetchx();
	TestHighScore();
	DispHighScores();

teacher:

	OSLinkClose();
// someday I'll get rid of those pesky global variables so I don't have to do this
	buffer[0] = 0; timer = 10; counter = 0; x = 0; y = 0; twoplay = 0; movedelay = 0,rotatedelay = 0; gameover = 0; intcounter = 0;  //set all values for next time, because they all retain their values!
	return;
};
